import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 2
key = xcffib.ExtensionKey("MIT-SHM")
_events = {}
_errors = {}
from . import xproto


class CompletionEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        self.drawable, self.minor_event, self.major_event, self.shmseg, self.offset = (
            unpacker.unpack("=xx2xIHBxII")
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack(
                "=x2xIHBxII",
                self.drawable,
                self.minor_event,
                self.major_event,
                self.shmseg,
                self.offset,
            )
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, drawable, minor_event, major_event, shmseg, offset):
        self = cls.__new__(cls)
        self.drawable = drawable
        self.minor_event = minor_event
        self.major_event = major_event
        self.shmseg = shmseg
        self.offset = offset
        return self


_events[0] = CompletionEvent


class BadSegError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        self.bad_value, self.minor_opcode, self.major_opcode = unpacker.unpack(
            "=xx2xIHBx"
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack(
                "=x2xIHBx", self.bad_value, self.minor_opcode, self.major_opcode
            )
        )
        return buf.getvalue()


BadBadSeg = BadSegError
_errors[0] = BadSegError


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.shared_pixmaps,
            self.major_version,
            self.minor_version,
            self.uid,
            self.gid,
            self.pixmap_format,
        ) = unpacker.unpack("=xB2x4xHHHHB15x")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class GetImageReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.depth, self.visual, self.size = unpacker.unpack("=xB2x4xII")
        self.bufsize = unpacker.offset - base


class GetImageCookie(xcffib.Cookie):
    reply_type = GetImageReply


class CreateSegmentReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.nfd,) = unpacker.unpack("=xB2x4x24x")
        self.bufsize = unpacker.offset - base


class CreateSegmentCookie(xcffib.Cookie):
    reply_type = CreateSegmentReply


class shmExtension(xcffib.Extension):
    def QueryVersion(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self):
        return self.QueryVersion(is_checked=True)

    def QueryVersionUnchecked(self):
        return self.QueryVersion(is_checked=False)

    def Attach(self, shmseg, shmid, read_only, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIB3x", shmseg, shmid, read_only))
        return self.send_request(1, buf, is_checked=is_checked)

    def AttachChecked(self, shmseg, shmid, read_only):
        return self.Attach(shmseg, shmid, read_only, is_checked=True)

    def AttachUnchecked(self, shmseg, shmid, read_only):
        return self.Attach(shmseg, shmid, read_only, is_checked=False)

    def Detach(self, shmseg, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", shmseg))
        return self.send_request(2, buf, is_checked=is_checked)

    def DetachChecked(self, shmseg):
        return self.Detach(shmseg, is_checked=True)

    def DetachUnchecked(self, shmseg):
        return self.Detach(shmseg, is_checked=False)

    def PutImage(
        self,
        drawable,
        gc,
        total_width,
        total_height,
        src_x,
        src_y,
        src_width,
        src_height,
        dst_x,
        dst_y,
        depth,
        format,
        send_event,
        shmseg,
        offset,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHHHHHhhBBBxII",
                drawable,
                gc,
                total_width,
                total_height,
                src_x,
                src_y,
                src_width,
                src_height,
                dst_x,
                dst_y,
                depth,
                format,
                send_event,
                shmseg,
                offset,
            )
        )
        return self.send_request(3, buf, is_checked=is_checked)

    def PutImageChecked(
        self,
        drawable,
        gc,
        total_width,
        total_height,
        src_x,
        src_y,
        src_width,
        src_height,
        dst_x,
        dst_y,
        depth,
        format,
        send_event,
        shmseg,
        offset,
    ):
        return self.PutImage(
            drawable,
            gc,
            total_width,
            total_height,
            src_x,
            src_y,
            src_width,
            src_height,
            dst_x,
            dst_y,
            depth,
            format,
            send_event,
            shmseg,
            offset,
            is_checked=True,
        )

    def PutImageUnchecked(
        self,
        drawable,
        gc,
        total_width,
        total_height,
        src_x,
        src_y,
        src_width,
        src_height,
        dst_x,
        dst_y,
        depth,
        format,
        send_event,
        shmseg,
        offset,
    ):
        return self.PutImage(
            drawable,
            gc,
            total_width,
            total_height,
            src_x,
            src_y,
            src_width,
            src_height,
            dst_x,
            dst_y,
            depth,
            format,
            send_event,
            shmseg,
            offset,
            is_checked=False,
        )

    def GetImage(
        self,
        drawable,
        x,
        y,
        width,
        height,
        plane_mask,
        format,
        shmseg,
        offset,
        is_checked=True,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIhhHHIB3xII",
                drawable,
                x,
                y,
                width,
                height,
                plane_mask,
                format,
                shmseg,
                offset,
            )
        )
        return self.send_request(4, buf, GetImageCookie, is_checked=is_checked)

    def GetImageChecked(
        self, drawable, x, y, width, height, plane_mask, format, shmseg, offset
    ):
        return self.GetImage(
            drawable,
            x,
            y,
            width,
            height,
            plane_mask,
            format,
            shmseg,
            offset,
            is_checked=True,
        )

    def GetImageUnchecked(
        self, drawable, x, y, width, height, plane_mask, format, shmseg, offset
    ):
        return self.GetImage(
            drawable,
            x,
            y,
            width,
            height,
            plane_mask,
            format,
            shmseg,
            offset,
            is_checked=False,
        )

    def CreatePixmap(
        self, pid, drawable, width, height, depth, shmseg, offset, is_checked=False
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIHHB3xII", pid, drawable, width, height, depth, shmseg, offset
            )
        )
        return self.send_request(5, buf, is_checked=is_checked)

    def CreatePixmapChecked(self, pid, drawable, width, height, depth, shmseg, offset):
        return self.CreatePixmap(
            pid, drawable, width, height, depth, shmseg, offset, is_checked=True
        )

    def CreatePixmapUnchecked(
        self, pid, drawable, width, height, depth, shmseg, offset
    ):
        return self.CreatePixmap(
            pid, drawable, width, height, depth, shmseg, offset, is_checked=False
        )

    def AttachFd(self, shmseg, read_only, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIB3x", shmseg, read_only))
        return self.send_request(6, buf, is_checked=is_checked)

    def AttachFdChecked(self, shmseg, read_only):
        return self.AttachFd(shmseg, read_only, is_checked=True)

    def AttachFdUnchecked(self, shmseg, read_only):
        return self.AttachFd(shmseg, read_only, is_checked=False)

    def CreateSegment(self, shmseg, size, read_only, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIB3x", shmseg, size, read_only))
        return self.send_request(7, buf, CreateSegmentCookie, is_checked=is_checked)

    def CreateSegmentChecked(self, shmseg, size, read_only):
        return self.CreateSegment(shmseg, size, read_only, is_checked=True)

    def CreateSegmentUnchecked(self, shmseg, size, read_only):
        return self.CreateSegment(shmseg, size, read_only, is_checked=False)


xcffib._add_ext(key, shmExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 2
MINOR_VERSION = 2
key = xcffib.ExtensionKey("XVideo")
_events = {}
_errors = {}
from . import xproto
from . import shm


class Type:
    InputMask = 1 << 0
    OutputMask = 1 << 1
    VideoMask = 1 << 2
    StillMask = 1 << 3
    ImageMask = 1 << 4


class ImageFormatInfoType:
    RGB = 0
    YUV = 1


class ImageFormatInfoFormat:
    Packed = 0
    Planar = 1


class AttributeFlag:
    Gettable = 1 << 0
    Settable = 1 << 1


class VideoNotifyReason:
    Started = 0
    Stopped = 1
    Busy = 2
    Preempted = 3
    HardError = 4


class ScanlineOrder:
    TopToBottom = 0
    BottomToTop = 1


class GrabPortStatus:
    Success = 0
    BadExtension = 1
    AlreadyGrabbed = 2
    InvalidTime = 3
    BadReply = 4
    BadAlloc = 5


class Rational(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.numerator, self.denominator = unpacker.unpack("=ii")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=ii", self.numerator, self.denominator))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, numerator, denominator):
        self = cls.__new__(cls)
        self.numerator = numerator
        self.denominator = denominator
        return self


class Format(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.visual, self.depth = unpacker.unpack("=IB3x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=IB3x", self.visual, self.depth))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, visual, depth):
        self = cls.__new__(cls)
        self.visual = visual
        self.depth = depth
        return self


class AdaptorInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.base_id, self.name_size, self.num_ports, self.num_formats, self.type = (
            unpacker.unpack("=IHHHBx")
        )
        self.name = xcffib.List(unpacker, "c", self.name_size)
        unpacker.pad(Format)
        self.formats = xcffib.List(unpacker, Format, self.num_formats)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IHHHBx",
                self.base_id,
                self.name_size,
                self.num_ports,
                self.num_formats,
                self.type,
            )
        )
        buf.write(xcffib.pack_list(self.name, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        buf.write(xcffib.pack_list(self.formats, Format))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, base_id, name_size, num_ports, num_formats, type, name, formats):
        self = cls.__new__(cls)
        self.base_id = base_id
        self.name_size = name_size
        self.num_ports = num_ports
        self.num_formats = num_formats
        self.type = type
        self.name = name
        self.formats = formats
        return self


class EncodingInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.encoding, self.name_size, self.width, self.height = unpacker.unpack(
            "=IHHH2x"
        )
        self.rate = Rational(unpacker)
        unpacker.pad("c")
        self.name = xcffib.List(unpacker, "c", self.name_size)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IHHH2x", self.encoding, self.name_size, self.width, self.height
            )
        )
        buf.write(
            self.rate.pack()
            if hasattr(self.rate, "pack")
            else Rational.synthetic(*self.rate).pack()
        )
        buf.write(xcffib.pack_list(self.name, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, encoding, name_size, width, height, rate, name):
        self = cls.__new__(cls)
        self.encoding = encoding
        self.name_size = name_size
        self.width = width
        self.height = height
        self.rate = rate
        self.name = name
        return self


class Image(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.id, self.width, self.height, self.data_size, self.num_planes = (
            unpacker.unpack("=IHHII")
        )
        self.pitches = xcffib.List(unpacker, "I", self.num_planes)
        unpacker.pad("I")
        self.offsets = xcffib.List(unpacker, "I", self.num_planes)
        unpacker.pad("B")
        self.data = xcffib.List(unpacker, "B", self.data_size)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IHHII",
                self.id,
                self.width,
                self.height,
                self.data_size,
                self.num_planes,
            )
        )
        buf.write(xcffib.pack_list(self.pitches, "I"))
        buf.write(xcffib.pack_list(self.offsets, "I"))
        buf.write(xcffib.pack_list(self.data, "B"))
        return buf.getvalue()

    @classmethod
    def synthetic(
        cls, id, width, height, data_size, num_planes, pitches, offsets, data
    ):
        self = cls.__new__(cls)
        self.id = id
        self.width = width
        self.height = height
        self.data_size = data_size
        self.num_planes = num_planes
        self.pitches = pitches
        self.offsets = offsets
        self.data = data
        return self


class AttributeInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.flags, self.min, self.max, self.size = unpacker.unpack("=IiiI")
        self.name = xcffib.List(unpacker, "c", self.size)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=IiiI", self.flags, self.min, self.max, self.size))
        buf.write(xcffib.pack_list(self.name, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        return buf.getvalue()

    @classmethod
    def synthetic(cls, flags, min, max, size, name):
        self = cls.__new__(cls)
        self.flags = flags
        self.min = min
        self.max = max
        self.size = size
        self.name = name
        return self


class ImageFormatInfo(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.id, self.type, self.byte_order = unpacker.unpack("=IBB2x")
        self.guid = xcffib.List(unpacker, "B", 16)
        (
            self.bpp,
            self.num_planes,
            self.depth,
            self.red_mask,
            self.green_mask,
            self.blue_mask,
            self.format,
            self.y_sample_bits,
            self.u_sample_bits,
            self.v_sample_bits,
            self.vhorz_y_period,
            self.vhorz_u_period,
            self.vhorz_v_period,
            self.vvert_y_period,
            self.vvert_u_period,
            self.vvert_v_period,
        ) = unpacker.unpack("=BB2xB3xIIIB3xIIIIIIIII")
        unpacker.pad("B")
        self.vcomp_order = xcffib.List(unpacker, "B", 32)
        (self.vscanline_order,) = unpacker.unpack("=B11x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=IBB2x", self.id, self.type, self.byte_order))
        buf.write(xcffib.pack_list(self.guid, "B"))
        buf.write(struct.pack("=B", self.bpp))
        buf.write(struct.pack("=B", self.num_planes))
        buf.write(
            struct.pack(
                "=2x",
            )
        )
        buf.write(struct.pack("=B", self.depth))
        buf.write(
            struct.pack(
                "=3x",
            )
        )
        buf.write(struct.pack("=I", self.red_mask))
        buf.write(struct.pack("=I", self.green_mask))
        buf.write(struct.pack("=I", self.blue_mask))
        buf.write(struct.pack("=B", self.format))
        buf.write(
            struct.pack(
                "=3x",
            )
        )
        buf.write(struct.pack("=I", self.y_sample_bits))
        buf.write(struct.pack("=I", self.u_sample_bits))
        buf.write(struct.pack("=I", self.v_sample_bits))
        buf.write(struct.pack("=I", self.vhorz_y_period))
        buf.write(struct.pack("=I", self.vhorz_u_period))
        buf.write(struct.pack("=I", self.vhorz_v_period))
        buf.write(struct.pack("=I", self.vvert_y_period))
        buf.write(struct.pack("=I", self.vvert_u_period))
        buf.write(struct.pack("=I", self.vvert_v_period))
        buf.write(xcffib.pack_list(self.vcomp_order, "B"))
        buf.write(struct.pack("=B", self.vscanline_order))
        buf.write(
            struct.pack(
                "=11x",
            )
        )
        return buf.getvalue()

    fixed_size = 128

    @classmethod
    def synthetic(
        cls,
        id,
        type,
        byte_order,
        guid,
        bpp,
        num_planes,
        depth,
        red_mask,
        green_mask,
        blue_mask,
        format,
        y_sample_bits,
        u_sample_bits,
        v_sample_bits,
        vhorz_y_period,
        vhorz_u_period,
        vhorz_v_period,
        vvert_y_period,
        vvert_u_period,
        vvert_v_period,
        vcomp_order,
        vscanline_order,
    ):
        self = cls.__new__(cls)
        self.id = id
        self.type = type
        self.byte_order = byte_order
        self.guid = guid
        self.bpp = bpp
        self.num_planes = num_planes
        self.depth = depth
        self.red_mask = red_mask
        self.green_mask = green_mask
        self.blue_mask = blue_mask
        self.format = format
        self.y_sample_bits = y_sample_bits
        self.u_sample_bits = u_sample_bits
        self.v_sample_bits = v_sample_bits
        self.vhorz_y_period = vhorz_y_period
        self.vhorz_u_period = vhorz_u_period
        self.vhorz_v_period = vhorz_v_period
        self.vvert_y_period = vvert_y_period
        self.vvert_u_period = vvert_u_period
        self.vvert_v_period = vvert_v_period
        self.vcomp_order = vcomp_order
        self.vscanline_order = vscanline_order
        return self


class BadPortError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadPort = BadPortError
_errors[0] = BadPortError


class BadEncodingError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 1))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadEncoding = BadEncodingError
_errors[1] = BadEncodingError


class BadControlError(xcffib.Error):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Error.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 2))
        buf.write(struct.pack("=x2x"))
        return buf.getvalue()


BadBadControl = BadControlError
_errors[2] = BadControlError


class VideoNotifyEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        self.reason, self.time, self.drawable, self.port = unpacker.unpack("=xB2xIII")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack("=B2xIII", self.reason, self.time, self.drawable, self.port)
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, reason, time, drawable, port):
        self = cls.__new__(cls)
        self.reason = reason
        self.time = time
        self.drawable = drawable
        self.port = port
        return self


_events[0] = VideoNotifyEvent


class PortNotifyEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        self.time, self.port, self.attribute, self.value = unpacker.unpack("=xx2xIIIi")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 1))
        buf.write(
            struct.pack("=x2xIIIi", self.time, self.port, self.attribute, self.value)
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, time, port, attribute, value):
        self = cls.__new__(cls)
        self.time = time
        self.port = port
        self.attribute = attribute
        self.value = value
        return self


_events[1] = PortNotifyEvent


class QueryExtensionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major, self.minor = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryExtensionCookie(xcffib.Cookie):
    reply_type = QueryExtensionReply


class QueryAdaptorsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.num_adaptors,) = unpacker.unpack("=xx2x4xH22x")
        self.info = xcffib.List(unpacker, AdaptorInfo, self.num_adaptors)
        self.bufsize = unpacker.offset - base


class QueryAdaptorsCookie(xcffib.Cookie):
    reply_type = QueryAdaptorsReply


class QueryEncodingsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.num_encodings,) = unpacker.unpack("=xx2x4xH22x")
        self.info = xcffib.List(unpacker, EncodingInfo, self.num_encodings)
        self.bufsize = unpacker.offset - base


class QueryEncodingsCookie(xcffib.Cookie):
    reply_type = QueryEncodingsReply


class GrabPortReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.result,) = unpacker.unpack("=xB2x4x")
        self.bufsize = unpacker.offset - base


class GrabPortCookie(xcffib.Cookie):
    reply_type = GrabPortReply


class QueryBestSizeReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.actual_width, self.actual_height = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryBestSizeCookie(xcffib.Cookie):
    reply_type = QueryBestSizeReply


class GetPortAttributeReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.value,) = unpacker.unpack("=xx2x4xi")
        self.bufsize = unpacker.offset - base


class GetPortAttributeCookie(xcffib.Cookie):
    reply_type = GetPortAttributeReply


class QueryPortAttributesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.num_attributes, self.text_size = unpacker.unpack("=xx2x4xII16x")
        self.attributes = xcffib.List(unpacker, AttributeInfo, self.num_attributes)
        self.bufsize = unpacker.offset - base


class QueryPortAttributesCookie(xcffib.Cookie):
    reply_type = QueryPortAttributesReply


class ListImageFormatsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.num_formats,) = unpacker.unpack("=xx2x4xI20x")
        self.format = xcffib.List(unpacker, ImageFormatInfo, self.num_formats)
        self.bufsize = unpacker.offset - base


class ListImageFormatsCookie(xcffib.Cookie):
    reply_type = ListImageFormatsReply


class QueryImageAttributesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.num_planes, self.data_size, self.width, self.height = unpacker.unpack(
            "=xx2x4xIIHH12x"
        )
        self.pitches = xcffib.List(unpacker, "I", self.num_planes)
        unpacker.pad("I")
        self.offsets = xcffib.List(unpacker, "I", self.num_planes)
        self.bufsize = unpacker.offset - base


class QueryImageAttributesCookie(xcffib.Cookie):
    reply_type = QueryImageAttributesReply


class xvExtension(xcffib.Extension):
    def QueryExtension(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(0, buf, QueryExtensionCookie, is_checked=is_checked)

    def QueryExtensionChecked(self):
        return self.QueryExtension(is_checked=True)

    def QueryExtensionUnchecked(self):
        return self.QueryExtension(is_checked=False)

    def QueryAdaptors(self, window, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", window))
        return self.send_request(1, buf, QueryAdaptorsCookie, is_checked=is_checked)

    def QueryAdaptorsChecked(self, window):
        return self.QueryAdaptors(window, is_checked=True)

    def QueryAdaptorsUnchecked(self, window):
        return self.QueryAdaptors(window, is_checked=False)

    def QueryEncodings(self, port, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", port))
        return self.send_request(2, buf, QueryEncodingsCookie, is_checked=is_checked)

    def QueryEncodingsChecked(self, port):
        return self.QueryEncodings(port, is_checked=True)

    def QueryEncodingsUnchecked(self, port):
        return self.QueryEncodings(port, is_checked=False)

    def GrabPort(self, port, time, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", port, time))
        return self.send_request(3, buf, GrabPortCookie, is_checked=is_checked)

    def GrabPortChecked(self, port, time):
        return self.GrabPort(port, time, is_checked=True)

    def GrabPortUnchecked(self, port, time):
        return self.GrabPort(port, time, is_checked=False)

    def UngrabPort(self, port, time, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", port, time))
        return self.send_request(4, buf, is_checked=is_checked)

    def UngrabPortChecked(self, port, time):
        return self.UngrabPort(port, time, is_checked=True)

    def UngrabPortUnchecked(self, port, time):
        return self.UngrabPort(port, time, is_checked=False)

    def PutVideo(
        self,
        port,
        drawable,
        gc,
        vid_x,
        vid_y,
        vid_w,
        vid_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIhhHHhhHH",
                port,
                drawable,
                gc,
                vid_x,
                vid_y,
                vid_w,
                vid_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
            )
        )
        return self.send_request(5, buf, is_checked=is_checked)

    def PutVideoChecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.PutVideo(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=True,
        )

    def PutVideoUnchecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.PutVideo(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=False,
        )

    def PutStill(
        self,
        port,
        drawable,
        gc,
        vid_x,
        vid_y,
        vid_w,
        vid_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIhhHHhhHH",
                port,
                drawable,
                gc,
                vid_x,
                vid_y,
                vid_w,
                vid_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
            )
        )
        return self.send_request(6, buf, is_checked=is_checked)

    def PutStillChecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.PutStill(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=True,
        )

    def PutStillUnchecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.PutStill(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=False,
        )

    def GetVideo(
        self,
        port,
        drawable,
        gc,
        vid_x,
        vid_y,
        vid_w,
        vid_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIhhHHhhHH",
                port,
                drawable,
                gc,
                vid_x,
                vid_y,
                vid_w,
                vid_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
            )
        )
        return self.send_request(7, buf, is_checked=is_checked)

    def GetVideoChecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.GetVideo(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=True,
        )

    def GetVideoUnchecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.GetVideo(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=False,
        )

    def GetStill(
        self,
        port,
        drawable,
        gc,
        vid_x,
        vid_y,
        vid_w,
        vid_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIhhHHhhHH",
                port,
                drawable,
                gc,
                vid_x,
                vid_y,
                vid_w,
                vid_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
            )
        )
        return self.send_request(8, buf, is_checked=is_checked)

    def GetStillChecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.GetStill(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=True,
        )

    def GetStillUnchecked(
        self, port, drawable, gc, vid_x, vid_y, vid_w, vid_h, drw_x, drw_y, drw_w, drw_h
    ):
        return self.GetStill(
            port,
            drawable,
            gc,
            vid_x,
            vid_y,
            vid_w,
            vid_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            is_checked=False,
        )

    def StopVideo(self, port, drawable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", port, drawable))
        return self.send_request(9, buf, is_checked=is_checked)

    def StopVideoChecked(self, port, drawable):
        return self.StopVideo(port, drawable, is_checked=True)

    def StopVideoUnchecked(self, port, drawable):
        return self.StopVideo(port, drawable, is_checked=False)

    def SelectVideoNotify(self, drawable, onoff, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIB3x", drawable, onoff))
        return self.send_request(10, buf, is_checked=is_checked)

    def SelectVideoNotifyChecked(self, drawable, onoff):
        return self.SelectVideoNotify(drawable, onoff, is_checked=True)

    def SelectVideoNotifyUnchecked(self, drawable, onoff):
        return self.SelectVideoNotify(drawable, onoff, is_checked=False)

    def SelectPortNotify(self, port, onoff, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIB3x", port, onoff))
        return self.send_request(11, buf, is_checked=is_checked)

    def SelectPortNotifyChecked(self, port, onoff):
        return self.SelectPortNotify(port, onoff, is_checked=True)

    def SelectPortNotifyUnchecked(self, port, onoff):
        return self.SelectPortNotify(port, onoff, is_checked=False)

    def QueryBestSize(self, port, vid_w, vid_h, drw_w, drw_h, motion, is_checked=True):
        buf = io.BytesIO()
        buf.write(
            struct.pack("=xx2xIHHHHB3x", port, vid_w, vid_h, drw_w, drw_h, motion)
        )
        return self.send_request(12, buf, QueryBestSizeCookie, is_checked=is_checked)

    def QueryBestSizeChecked(self, port, vid_w, vid_h, drw_w, drw_h, motion):
        return self.QueryBestSize(
            port, vid_w, vid_h, drw_w, drw_h, motion, is_checked=True
        )

    def QueryBestSizeUnchecked(self, port, vid_w, vid_h, drw_w, drw_h, motion):
        return self.QueryBestSize(
            port, vid_w, vid_h, drw_w, drw_h, motion, is_checked=False
        )

    def SetPortAttribute(self, port, attribute, value, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIi", port, attribute, value))
        return self.send_request(13, buf, is_checked=is_checked)

    def SetPortAttributeChecked(self, port, attribute, value):
        return self.SetPortAttribute(port, attribute, value, is_checked=True)

    def SetPortAttributeUnchecked(self, port, attribute, value):
        return self.SetPortAttribute(port, attribute, value, is_checked=False)

    def GetPortAttribute(self, port, attribute, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", port, attribute))
        return self.send_request(14, buf, GetPortAttributeCookie, is_checked=is_checked)

    def GetPortAttributeChecked(self, port, attribute):
        return self.GetPortAttribute(port, attribute, is_checked=True)

    def GetPortAttributeUnchecked(self, port, attribute):
        return self.GetPortAttribute(port, attribute, is_checked=False)

    def QueryPortAttributes(self, port, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", port))
        return self.send_request(
            15, buf, QueryPortAttributesCookie, is_checked=is_checked
        )

    def QueryPortAttributesChecked(self, port):
        return self.QueryPortAttributes(port, is_checked=True)

    def QueryPortAttributesUnchecked(self, port):
        return self.QueryPortAttributes(port, is_checked=False)

    def ListImageFormats(self, port, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", port))
        return self.send_request(16, buf, ListImageFormatsCookie, is_checked=is_checked)

    def ListImageFormatsChecked(self, port):
        return self.ListImageFormats(port, is_checked=True)

    def ListImageFormatsUnchecked(self, port):
        return self.ListImageFormats(port, is_checked=False)

    def QueryImageAttributes(self, port, id, width, height, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIHH", port, id, width, height))
        return self.send_request(
            17, buf, QueryImageAttributesCookie, is_checked=is_checked
        )

    def QueryImageAttributesChecked(self, port, id, width, height):
        return self.QueryImageAttributes(port, id, width, height, is_checked=True)

    def QueryImageAttributesUnchecked(self, port, id, width, height):
        return self.QueryImageAttributes(port, id, width, height, is_checked=False)

    def PutImage(
        self,
        port,
        drawable,
        gc,
        id,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        data_len,
        data,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIIhhHHhhHHHH",
                port,
                drawable,
                gc,
                id,
                src_x,
                src_y,
                src_w,
                src_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
                width,
                height,
            )
        )
        buf.write(xcffib.pack_list(data, "B"))
        return self.send_request(18, buf, is_checked=is_checked)

    def PutImageChecked(
        self,
        port,
        drawable,
        gc,
        id,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        data_len,
        data,
    ):
        return self.PutImage(
            port,
            drawable,
            gc,
            id,
            src_x,
            src_y,
            src_w,
            src_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            width,
            height,
            data_len,
            data,
            is_checked=True,
        )

    def PutImageUnchecked(
        self,
        port,
        drawable,
        gc,
        id,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        data_len,
        data,
    ):
        return self.PutImage(
            port,
            drawable,
            gc,
            id,
            src_x,
            src_y,
            src_w,
            src_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            width,
            height,
            data_len,
            data,
            is_checked=False,
        )

    def ShmPutImage(
        self,
        port,
        drawable,
        gc,
        shmseg,
        id,
        offset,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        send_event,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIIIIhhHHhhHHHHB3x",
                port,
                drawable,
                gc,
                shmseg,
                id,
                offset,
                src_x,
                src_y,
                src_w,
                src_h,
                drw_x,
                drw_y,
                drw_w,
                drw_h,
                width,
                height,
                send_event,
            )
        )
        return self.send_request(19, buf, is_checked=is_checked)

    def ShmPutImageChecked(
        self,
        port,
        drawable,
        gc,
        shmseg,
        id,
        offset,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        send_event,
    ):
        return self.ShmPutImage(
            port,
            drawable,
            gc,
            shmseg,
            id,
            offset,
            src_x,
            src_y,
            src_w,
            src_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            width,
            height,
            send_event,
            is_checked=True,
        )

    def ShmPutImageUnchecked(
        self,
        port,
        drawable,
        gc,
        shmseg,
        id,
        offset,
        src_x,
        src_y,
        src_w,
        src_h,
        drw_x,
        drw_y,
        drw_w,
        drw_h,
        width,
        height,
        send_event,
    ):
        return self.ShmPutImage(
            port,
            drawable,
            gc,
            shmseg,
            id,
            offset,
            src_x,
            src_y,
            src_w,
            src_h,
            drw_x,
            drw_y,
            drw_w,
            drw_h,
            width,
            height,
            send_event,
            is_checked=False,
        )


xcffib._add_ext(key, xvExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 0
key = xcffib.ExtensionKey("XEVIE")
_events = {}
_errors = {}


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.server_major_version, self.server_minor_version = unpacker.unpack(
            "=xx2x4xHH20x"
        )
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class StartReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x4x24x")
        self.bufsize = unpacker.offset - base


class StartCookie(xcffib.Cookie):
    reply_type = StartReply


class EndReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x4x24x")
        self.bufsize = unpacker.offset - base


class EndCookie(xcffib.Cookie):
    reply_type = EndReply


class Datatype:
    Unmodified = 0
    Modified = 1


class Event(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=32x")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=32x"))
        return buf.getvalue()

    fixed_size = 32


class SendReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x4x24x")
        self.bufsize = unpacker.offset - base


class SendCookie(xcffib.Cookie):
    reply_type = SendReply


class SelectInputReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x4x24x")
        self.bufsize = unpacker.offset - base


class SelectInputCookie(xcffib.Cookie):
    reply_type = SelectInputReply


class xevieExtension(xcffib.Extension):
    def QueryVersion(self, client_major_version, client_minor_version, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xHH", client_major_version, client_minor_version))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=True
        )

    def QueryVersionUnchecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=False
        )

    def Start(self, screen, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", screen))
        return self.send_request(1, buf, StartCookie, is_checked=is_checked)

    def StartChecked(self, screen):
        return self.Start(screen, is_checked=True)

    def StartUnchecked(self, screen):
        return self.Start(screen, is_checked=False)

    def End(self, cmap, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", cmap))
        return self.send_request(2, buf, EndCookie, is_checked=is_checked)

    def EndChecked(self, cmap):
        return self.End(cmap, is_checked=True)

    def EndUnchecked(self, cmap):
        return self.End(cmap, is_checked=False)

    def Send(self, event, data_type, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        buf.write(
            event.pack() if hasattr(event, "pack") else Event.synthetic(*event).pack()
        )
        buf.write(struct.pack("=I", data_type))
        buf.write(
            struct.pack(
                "=64x",
            )
        )
        return self.send_request(3, buf, SendCookie, is_checked=is_checked)

    def SendChecked(self, event, data_type):
        return self.Send(event, data_type, is_checked=True)

    def SendUnchecked(self, event, data_type):
        return self.Send(event, data_type, is_checked=False)

    def SelectInput(self, event_mask, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", event_mask))
        return self.send_request(4, buf, SelectInputCookie, is_checked=is_checked)

    def SelectInputChecked(self, event_mask):
        return self.SelectInput(event_mask, is_checked=True)

    def SelectInputUnchecked(self, event_mask):
        return self.SelectInput(event_mask, is_checked=False)


xcffib._add_ext(key, xevieExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 4
key = xcffib.ExtensionKey("DRI2")
_events = {}
_errors = {}
from . import xproto


class Attachment:
    BufferFrontLeft = 0
    BufferBackLeft = 1
    BufferFrontRight = 2
    BufferBackRight = 3
    BufferDepth = 4
    BufferStencil = 5
    BufferAccum = 6
    BufferFakeFrontLeft = 7
    BufferFakeFrontRight = 8
    BufferDepthStencil = 9
    BufferHiz = 10


class DriverType:
    DRI = 0
    VDPAU = 1


class EventType:
    ExchangeComplete = 1
    BlitComplete = 2
    FlipComplete = 3


class DRI2Buffer(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.attachment, self.name, self.pitch, self.cpp, self.flags = unpacker.unpack(
            "=IIIII"
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=IIIII", self.attachment, self.name, self.pitch, self.cpp, self.flags
            )
        )
        return buf.getvalue()

    fixed_size = 20

    @classmethod
    def synthetic(cls, attachment, name, pitch, cpp, flags):
        self = cls.__new__(cls)
        self.attachment = attachment
        self.name = name
        self.pitch = pitch
        self.cpp = cpp
        self.flags = flags
        return self


class AttachFormat(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.attachment, self.format = unpacker.unpack("=II")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=II", self.attachment, self.format))
        return buf.getvalue()

    fixed_size = 8

    @classmethod
    def synthetic(cls, attachment, format):
        self = cls.__new__(cls)
        self.attachment = attachment
        self.format = format
        return self


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.major_version, self.minor_version = unpacker.unpack("=xx2x4xII")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class ConnectReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.driver_name_length, self.device_name_length = unpacker.unpack(
            "=xx2x4xII16x"
        )
        self.driver_name = xcffib.List(unpacker, "c", self.driver_name_length)
        unpacker.pad("c")
        self.alignment_pad = xcffib.List(
            unpacker,
            "c",
            ((self.driver_name_length + 3) & (~3)) - self.driver_name_length,
        )
        unpacker.pad("c")
        self.device_name = xcffib.List(unpacker, "c", self.device_name_length)
        self.bufsize = unpacker.offset - base


class ConnectCookie(xcffib.Cookie):
    reply_type = ConnectReply


class AuthenticateReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.authenticated,) = unpacker.unpack("=xx2x4xI")
        self.bufsize = unpacker.offset - base


class AuthenticateCookie(xcffib.Cookie):
    reply_type = AuthenticateReply


class GetBuffersReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.width, self.height, self.count = unpacker.unpack("=xx2x4xIII12x")
        self.buffers = xcffib.List(unpacker, DRI2Buffer, self.count)
        self.bufsize = unpacker.offset - base


class GetBuffersCookie(xcffib.Cookie):
    reply_type = GetBuffersReply


class CopyRegionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        unpacker.unpack("=xx2x4x")
        self.bufsize = unpacker.offset - base


class CopyRegionCookie(xcffib.Cookie):
    reply_type = CopyRegionReply


class GetBuffersWithFormatReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.width, self.height, self.count = unpacker.unpack("=xx2x4xIII12x")
        self.buffers = xcffib.List(unpacker, DRI2Buffer, self.count)
        self.bufsize = unpacker.offset - base


class GetBuffersWithFormatCookie(xcffib.Cookie):
    reply_type = GetBuffersWithFormatReply


class SwapBuffersReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.swap_hi, self.swap_lo = unpacker.unpack("=xx2x4xII")
        self.bufsize = unpacker.offset - base


class SwapBuffersCookie(xcffib.Cookie):
    reply_type = SwapBuffersReply


class GetMSCReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.ust_hi, self.ust_lo, self.msc_hi, self.msc_lo, self.sbc_hi, self.sbc_lo = (
            unpacker.unpack("=xx2x4xIIIIII")
        )
        self.bufsize = unpacker.offset - base


class GetMSCCookie(xcffib.Cookie):
    reply_type = GetMSCReply


class WaitMSCReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.ust_hi, self.ust_lo, self.msc_hi, self.msc_lo, self.sbc_hi, self.sbc_lo = (
            unpacker.unpack("=xx2x4xIIIIII")
        )
        self.bufsize = unpacker.offset - base


class WaitMSCCookie(xcffib.Cookie):
    reply_type = WaitMSCReply


class WaitSBCReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.ust_hi, self.ust_lo, self.msc_hi, self.msc_lo, self.sbc_hi, self.sbc_lo = (
            unpacker.unpack("=xx2x4xIIIIII")
        )
        self.bufsize = unpacker.offset - base


class WaitSBCCookie(xcffib.Cookie):
    reply_type = WaitSBCReply


class GetParamReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.is_param_recognized, self.value_hi, self.value_lo = unpacker.unpack(
            "=xB2x4xII"
        )
        self.bufsize = unpacker.offset - base


class GetParamCookie(xcffib.Cookie):
    reply_type = GetParamReply


class BufferSwapCompleteEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.event_type,
            self.drawable,
            self.ust_hi,
            self.ust_lo,
            self.msc_hi,
            self.msc_lo,
            self.sbc,
        ) = unpacker.unpack("=xx2xH2xIIIIII")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack(
                "=x2xH2xIIIIII",
                self.event_type,
                self.drawable,
                self.ust_hi,
                self.ust_lo,
                self.msc_hi,
                self.msc_lo,
                self.sbc,
            )
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, event_type, drawable, ust_hi, ust_lo, msc_hi, msc_lo, sbc):
        self = cls.__new__(cls)
        self.event_type = event_type
        self.drawable = drawable
        self.ust_hi = ust_hi
        self.ust_lo = ust_lo
        self.msc_hi = msc_hi
        self.msc_lo = msc_lo
        self.sbc = sbc
        return self


_events[0] = BufferSwapCompleteEvent


class InvalidateBuffersEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        (self.drawable,) = unpacker.unpack("=xx2xI")
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 1))
        buf.write(struct.pack("=x2xI", self.drawable))
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, drawable):
        self = cls.__new__(cls)
        self.drawable = drawable
        return self


_events[1] = InvalidateBuffersEvent


class dri2Extension(xcffib.Extension):
    def QueryVersion(self, major_version, minor_version, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", major_version, minor_version))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, major_version, minor_version):
        return self.QueryVersion(major_version, minor_version, is_checked=True)

    def QueryVersionUnchecked(self, major_version, minor_version):
        return self.QueryVersion(major_version, minor_version, is_checked=False)

    def Connect(self, window, driver_type, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", window, driver_type))
        return self.send_request(1, buf, ConnectCookie, is_checked=is_checked)

    def ConnectChecked(self, window, driver_type):
        return self.Connect(window, driver_type, is_checked=True)

    def ConnectUnchecked(self, window, driver_type):
        return self.Connect(window, driver_type, is_checked=False)

    def Authenticate(self, window, magic, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", window, magic))
        return self.send_request(2, buf, AuthenticateCookie, is_checked=is_checked)

    def AuthenticateChecked(self, window, magic):
        return self.Authenticate(window, magic, is_checked=True)

    def AuthenticateUnchecked(self, window, magic):
        return self.Authenticate(window, magic, is_checked=False)

    def CreateDrawable(self, drawable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(3, buf, is_checked=is_checked)

    def CreateDrawableChecked(self, drawable):
        return self.CreateDrawable(drawable, is_checked=True)

    def CreateDrawableUnchecked(self, drawable):
        return self.CreateDrawable(drawable, is_checked=False)

    def DestroyDrawable(self, drawable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(4, buf, is_checked=is_checked)

    def DestroyDrawableChecked(self, drawable):
        return self.DestroyDrawable(drawable, is_checked=True)

    def DestroyDrawableUnchecked(self, drawable):
        return self.DestroyDrawable(drawable, is_checked=False)

    def GetBuffers(
        self, drawable, count, attachments_len, attachments, is_checked=True
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", drawable, count))
        buf.write(xcffib.pack_list(attachments, "I"))
        return self.send_request(5, buf, GetBuffersCookie, is_checked=is_checked)

    def GetBuffersChecked(self, drawable, count, attachments_len, attachments):
        return self.GetBuffers(
            drawable, count, attachments_len, attachments, is_checked=True
        )

    def GetBuffersUnchecked(self, drawable, count, attachments_len, attachments):
        return self.GetBuffers(
            drawable, count, attachments_len, attachments, is_checked=False
        )

    def CopyRegion(self, drawable, region, dest, src, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIIII", drawable, region, dest, src))
        return self.send_request(6, buf, CopyRegionCookie, is_checked=is_checked)

    def CopyRegionChecked(self, drawable, region, dest, src):
        return self.CopyRegion(drawable, region, dest, src, is_checked=True)

    def CopyRegionUnchecked(self, drawable, region, dest, src):
        return self.CopyRegion(drawable, region, dest, src, is_checked=False)

    def GetBuffersWithFormat(
        self, drawable, count, attachments_len, attachments, is_checked=True
    ):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", drawable, count))
        buf.write(xcffib.pack_list(attachments, AttachFormat))
        return self.send_request(
            7, buf, GetBuffersWithFormatCookie, is_checked=is_checked
        )

    def GetBuffersWithFormatChecked(
        self, drawable, count, attachments_len, attachments
    ):
        return self.GetBuffersWithFormat(
            drawable, count, attachments_len, attachments, is_checked=True
        )

    def GetBuffersWithFormatUnchecked(
        self, drawable, count, attachments_len, attachments
    ):
        return self.GetBuffersWithFormat(
            drawable, count, attachments_len, attachments, is_checked=False
        )

    def SwapBuffers(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
        is_checked=True,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIIIII",
                drawable,
                target_msc_hi,
                target_msc_lo,
                divisor_hi,
                divisor_lo,
                remainder_hi,
                remainder_lo,
            )
        )
        return self.send_request(8, buf, SwapBuffersCookie, is_checked=is_checked)

    def SwapBuffersChecked(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
    ):
        return self.SwapBuffers(
            drawable,
            target_msc_hi,
            target_msc_lo,
            divisor_hi,
            divisor_lo,
            remainder_hi,
            remainder_lo,
            is_checked=True,
        )

    def SwapBuffersUnchecked(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
    ):
        return self.SwapBuffers(
            drawable,
            target_msc_hi,
            target_msc_lo,
            divisor_hi,
            divisor_lo,
            remainder_hi,
            remainder_lo,
            is_checked=False,
        )

    def GetMSC(self, drawable, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(9, buf, GetMSCCookie, is_checked=is_checked)

    def GetMSCChecked(self, drawable):
        return self.GetMSC(drawable, is_checked=True)

    def GetMSCUnchecked(self, drawable):
        return self.GetMSC(drawable, is_checked=False)

    def WaitMSC(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
        is_checked=True,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIIIIIII",
                drawable,
                target_msc_hi,
                target_msc_lo,
                divisor_hi,
                divisor_lo,
                remainder_hi,
                remainder_lo,
            )
        )
        return self.send_request(10, buf, WaitMSCCookie, is_checked=is_checked)

    def WaitMSCChecked(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
    ):
        return self.WaitMSC(
            drawable,
            target_msc_hi,
            target_msc_lo,
            divisor_hi,
            divisor_lo,
            remainder_hi,
            remainder_lo,
            is_checked=True,
        )

    def WaitMSCUnchecked(
        self,
        drawable,
        target_msc_hi,
        target_msc_lo,
        divisor_hi,
        divisor_lo,
        remainder_hi,
        remainder_lo,
    ):
        return self.WaitMSC(
            drawable,
            target_msc_hi,
            target_msc_lo,
            divisor_hi,
            divisor_lo,
            remainder_hi,
            remainder_lo,
            is_checked=False,
        )

    def WaitSBC(self, drawable, target_sbc_hi, target_sbc_lo, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xIII", drawable, target_sbc_hi, target_sbc_lo))
        return self.send_request(11, buf, WaitSBCCookie, is_checked=is_checked)

    def WaitSBCChecked(self, drawable, target_sbc_hi, target_sbc_lo):
        return self.WaitSBC(drawable, target_sbc_hi, target_sbc_lo, is_checked=True)

    def WaitSBCUnchecked(self, drawable, target_sbc_hi, target_sbc_lo):
        return self.WaitSBC(drawable, target_sbc_hi, target_sbc_lo, is_checked=False)

    def SwapInterval(self, drawable, interval, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", drawable, interval))
        return self.send_request(12, buf, is_checked=is_checked)

    def SwapIntervalChecked(self, drawable, interval):
        return self.SwapInterval(drawable, interval, is_checked=True)

    def SwapIntervalUnchecked(self, drawable, interval):
        return self.SwapInterval(drawable, interval, is_checked=False)

    def GetParam(self, drawable, param, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", drawable, param))
        return self.send_request(13, buf, GetParamCookie, is_checked=is_checked)

    def GetParamChecked(self, drawable, param):
        return self.GetParam(drawable, param, is_checked=True)

    def GetParamUnchecked(self, drawable, param):
        return self.GetParam(drawable, param, is_checked=False)


xcffib._add_ext(key, dri2Extension, _events, _errors)



# Copyright 2014 Tycho Andersen
# Copyright 2014 Sean Vig
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import xcffib
import xcffib.xproto

from xcffib import ffi

import pytest


class TestConnection:
    def test_invalid_display(self, xproto_test):
        with pytest.raises(xcffib.ConnectionException):
            xproto_test.conn = xcffib.Connection("notvalid")

    def test_get_setup(self, xproto_test):
        setup = xproto_test.conn.get_setup()
        # When X upgrades, we can probably manage to change this test :-)
        assert setup.protocol_major_version == 11
        assert setup.protocol_minor_version == 0

    def test_get_screen_pointers(self, xproto_test):
        screens = xproto_test.conn.get_screen_pointers()
        assert len(screens) == 1
        screen = screens[0]
        assert ffi.typeof(screen) is ffi.typeof("xcb_screen_t *")
        assert screen.root == xproto_test.default_screen.root
        assert screen.width_in_pixels == xproto_test.width
        assert screen.height_in_pixels == xproto_test.height
        assert screen.root_depth == xproto_test.depth

    def test_discard_sequence(self, xproto_test):
        cookie = xproto_test.xproto.GetInputFocus()
        cookie.discard_reply()
        # this hangs if you leave it in, because the reply really was discarded
        # assert cookie.reply()

    def test_invalid(self, xproto_test):
        with pytest.raises(xcffib.ConnectionException):
            xcffib.Connection("notadisplay")

    def test_list_extensions(self, xproto_test):
        reply = xproto_test.conn.core.ListExtensions().reply()
        exts = [ext.name.to_string() for ext in reply.names]
        assert "XVideo" in exts

    def test_create_window(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        cookie = xproto_test.create_window(wid=wid)

        cookie = xproto_test.xproto.GetGeometry(wid)

        reply = cookie.reply()
        assert reply.x == 0
        assert reply.y == 0
        assert reply.width == 1
        assert reply.height == 1

    def test_wait_for_nonexistent_request(self, xproto_test):
        with pytest.raises(xcffib.XcffibException):
            xproto_test.conn.wait_for_reply(10)

    def test_no_windows(self, xproto_test):
        # Make sure there aren't any windows in the root window. This mostly
        # just exists to make sure people aren't somehow mistakenly running a
        # test in their own X session, which could corrupt results.
        reply = xproto_test.xproto.QueryTree(xproto_test.default_screen.root).reply()
        assert reply.children_len == 0
        assert len(reply.children) == 0

    def test_create_window_creates_window(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)
        reply = xproto_test.xproto.QueryTree(xproto_test.default_screen.root).reply()
        assert reply.children_len == 1
        assert len(reply.children) == 1
        assert reply.children[0] == wid

    def test_checking_unchecked_fails(self, xproto_test):
        with pytest.raises(AssertionError):
            wid = xproto_test.conn.generate_id()
            xproto_test.create_window(wid)
            xproto_test.xproto.QueryTreeUnchecked(
                xproto_test.default_screen.root
            ).check()

    def test_checking_default_checked_fails(self, xproto_test):
        with pytest.raises(AssertionError):
            wid = xproto_test.conn.generate_id()
            xproto_test.create_window(wid)
            cookie = xproto_test.xproto.QueryTree(xproto_test.default_screen.root)
            cookie.check()

    def test_checking_foreced_checked_succeeds(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        cookie = xproto_test.create_window(wid, is_checked=True)
        cookie.check()

    def test_create_window_generates_event(self, xproto_test):
        xproto_test.xeyes()

        e = xproto_test.conn.wait_for_event()
        assert isinstance(e, xcffib.xproto.CreateNotifyEvent)

    def test_query_invalid_wid_generates_error(self, xproto_test):
        with pytest.raises(xcffib.xproto.WindowError):
            # query a bad WINDOW
            xproto_test.xproto.QueryTree(0xF00).reply()

    def test_OpenFont(self, xproto_test):
        fid = xproto_test.conn.generate_id()
        xproto_test.xproto.OpenFont(fid, len("cursor"), "cursor")

    def test_ConfigureWindow(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)
        xproto_test.xproto.ConfigureWindowChecked(wid, 0, []).check()

    def test_external_ConfigureWindow(self, xproto_test):
        xproto_test.xeyes()

        e = xproto_test.conn.wait_for_event()

        xproto_test.xproto.ConfigureWindowChecked(e.window, 0, []).check()
        xproto_test.xproto.DestroyWindowChecked(e.window).check()

    def test_ChangeProperty_WM_NAME(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)

        title = "test"
        xproto_test.xproto.ChangeProperty(
            xcffib.xproto.PropMode.Replace,
            wid,
            xcffib.xproto.Atom.WM_NAME,
            xcffib.xproto.Atom.STRING,
            8,
            len(title),
            title,
        )

        reply = xproto_test.xproto.GetProperty(
            False,
            wid,
            xcffib.xproto.Atom.WM_NAME,
            xcffib.xproto.GetPropertyType.Any,
            0,
            1,
        ).reply()
        assert reply.value.to_string() == title

    def test_ChangeProperty_NET_WM_NAME(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)

        net_wm_name = xproto_test.intern("_NET_WM_NAME")
        utf8_string = xproto_test.intern("UTF8_STRING")

        title_bytes = b"test\xc2\xb7"
        title_string = "test\u00b7"

        # First check with an object already encoded as bytes
        xproto_test.xproto.ChangeProperty(
            xcffib.xproto.PropMode.Replace,
            wid,
            net_wm_name,
            utf8_string,
            8,
            len(title_bytes),
            title_bytes,
        )

        reply = xproto_test.xproto.GetProperty(
            False, wid, net_wm_name, xcffib.xproto.GetPropertyType.Any, 0, (2**32) - 1
        ).reply()

        print(reply.value.buf())
        assert reply.value.buf() == title_bytes
        print(reply.value.to_utf8())
        assert reply.value.to_utf8() == title_string

        # Also check with a unicode string
        xproto_test.xproto.ChangeProperty(
            xcffib.xproto.PropMode.Replace,
            wid,
            net_wm_name,
            utf8_string,
            8,
            len(title_string.encode("utf-8")),
            title_string,
        )

        reply = xproto_test.xproto.GetProperty(
            False, wid, net_wm_name, xcffib.xproto.GetPropertyType.Any, 0, (2**32) - 1
        ).reply()

        assert reply.value.buf() == title_bytes
        assert reply.value.to_utf8() == title_string

    def test_ChangeProperty_WM_PROTOCOLS(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)

        wm_protocols = xproto_test.intern("WM_PROTOCOLS")

        wm_delete_window = xproto_test.intern("WM_DELETE_WINDOW")

        xproto_test.xproto.ChangeProperty(
            xcffib.xproto.PropMode.Replace,
            wid,
            wm_protocols,
            xcffib.xproto.Atom.ATOM,
            32,
            1,
            (wm_delete_window,),
        )

        reply = xproto_test.xproto.GetProperty(
            False, wid, wm_protocols, xcffib.xproto.Atom.ATOM, 0, 1
        ).reply()

        assert reply.value.to_atoms() == (wm_delete_window,)

        wm_take_focus = xproto_test.intern("WM_TAKE_FOCUS")

        xproto_test.xproto.ChangeProperty(
            xcffib.xproto.PropMode.Replace,
            wid,
            wm_protocols,
            xcffib.xproto.Atom.ATOM,
            32,
            1,
            (wm_take_focus,),
        )

        reply = xproto_test.xproto.GetProperty(
            False, wid, wm_protocols, xcffib.xproto.Atom.ATOM, 0, 1
        ).reply()

        assert reply.value.to_atoms() == (wm_take_focus,)

    def test_GetAtomName(self, xproto_test):
        wm_protocols = "WM_PROTOCOLS"
        atom = xproto_test.intern(wm_protocols)
        atom_name = xproto_test.xproto.GetAtomName(atom).reply().name

        assert atom_name.to_string() == wm_protocols

    def test_KillClient(self, xproto_test):
        xproto_test.xeyes()

        e1 = xproto_test.conn.wait_for_event()
        xproto_test.xproto.KillClient(e1.window)

        # one is MapRequest and the other is DestroyNotify, they may be in
        # either order
        for _ in range(2):
            xproto_test.conn.flush()
            k1 = xproto_test.conn.wait_for_event()
            if isinstance(k1, xcffib.xproto.DestroyNotifyEvent):
                assert e1.window == k1.window
                return
        assert False, "no DestroyNotifyEvent"

    def test_connect(self, xproto_test):
        c = xcffib.connect()
        c.invalid()
        assert c.has_error() == 0
        c.disconnect()

    def test_auth_connect(self, xproto_test):
        authname = b"MIT-MAGIC-COOKIE-1"
        authdata = b"\xa5\xcf\x95\xfa\x19\x49\x03\x60\xaf\xe4\x1e\xcd\xa3\xe2\xad\x47"

        authstr = authname + b":" + authdata

        conn = xcffib.connect(display=os.environ["DISPLAY"], auth=authstr)

        assert conn.get_setup().roots[0].root > 0

    # This is an adaptation of the test from #27
    def test_build_atom_cache(self, xproto_test):
        # This will hold the forward *and* reverse lookups for any given atom
        atoms = {}
        cookies = []
        # Batch the replies by creating a list of cookies first:
        for i in range(1, 10000):
            c = xproto_test.conn.core.GetAtomName(i)
            cookies.append((i, c))
        for i, c in cookies:
            try:
                name = c.reply().name.to_string()
            except xcffib.xproto.BadAtom:
                continue
            atoms.update({i: name})  # Lookup by number
            atoms.update({name: i})  # Lookup by name

    def test_wrap(self, xproto_test):
        c = xcffib.connect()
        c.invalid()
        c2 = xcffib.wrap(xcffib.ffi.cast("long", c._conn))
        c2.invalid()
        c2.disconnect()

    def test_unmap_window_unchecked(self, xproto_test):
        wid = xproto_test.conn.generate_id()
        xproto_test.create_window(wid=wid)
        reply = xproto_test.xproto.QueryTree(xproto_test.default_screen.root).reply()
        assert wid in reply.children

        cookie = xproto_test.xproto.UnmapWindowUnchecked(wid)
        with pytest.raises(AssertionError):
            cookie.check()

        xproto_test.conn.flush()
        reply = xproto_test.xproto.QueryTree(xproto_test.default_screen.root).reply()
        assert wid in reply.children



import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 0
key = xcffib.ExtensionKey("SELinux")
_events = {}
_errors = {}
from . import xproto


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.server_major, self.server_minor = unpacker.unpack("=xx2x4xHH")
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class GetDeviceCreateContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetDeviceCreateContextCookie(xcffib.Cookie):
    reply_type = GetDeviceCreateContextReply


class GetDeviceContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetDeviceContextCookie(xcffib.Cookie):
    reply_type = GetDeviceContextReply


class GetWindowCreateContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetWindowCreateContextCookie(xcffib.Cookie):
    reply_type = GetWindowCreateContextReply


class GetWindowContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetWindowContextCookie(xcffib.Cookie):
    reply_type = GetWindowContextReply


class ListItem(xcffib.Struct):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Struct.__init__(self, unpacker)
        base = unpacker.offset
        self.name, self.object_context_len, self.data_context_len = unpacker.unpack(
            "=III"
        )
        self.object_context = xcffib.List(unpacker, "c", self.object_context_len)
        unpacker.pad("c")
        self.data_context = xcffib.List(unpacker, "c", self.data_context_len)
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=III", self.name, self.object_context_len, self.data_context_len
            )
        )
        buf.write(xcffib.pack_list(self.object_context, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        buf.write(xcffib.pack_list(self.data_context, "c"))
        buf.write(
            struct.pack(
                "=4x",
            )
        )
        return buf.getvalue()

    @classmethod
    def synthetic(
        cls, name, object_context_len, data_context_len, object_context, data_context
    ):
        self = cls.__new__(cls)
        self.name = name
        self.object_context_len = object_context_len
        self.data_context_len = data_context_len
        self.object_context = object_context
        self.data_context = data_context
        return self


class GetPropertyCreateContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetPropertyCreateContextCookie(xcffib.Cookie):
    reply_type = GetPropertyCreateContextReply


class GetPropertyUseContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetPropertyUseContextCookie(xcffib.Cookie):
    reply_type = GetPropertyUseContextReply


class GetPropertyContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetPropertyContextCookie(xcffib.Cookie):
    reply_type = GetPropertyContextReply


class GetPropertyDataContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetPropertyDataContextCookie(xcffib.Cookie):
    reply_type = GetPropertyDataContextReply


class ListPropertiesReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.properties_len,) = unpacker.unpack("=xx2x4xI20x")
        self.properties = xcffib.List(unpacker, ListItem, self.properties_len)
        self.bufsize = unpacker.offset - base


class ListPropertiesCookie(xcffib.Cookie):
    reply_type = ListPropertiesReply


class GetSelectionCreateContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetSelectionCreateContextCookie(xcffib.Cookie):
    reply_type = GetSelectionCreateContextReply


class GetSelectionUseContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetSelectionUseContextCookie(xcffib.Cookie):
    reply_type = GetSelectionUseContextReply


class GetSelectionContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetSelectionContextCookie(xcffib.Cookie):
    reply_type = GetSelectionContextReply


class GetSelectionDataContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetSelectionDataContextCookie(xcffib.Cookie):
    reply_type = GetSelectionDataContextReply


class ListSelectionsReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.selections_len,) = unpacker.unpack("=xx2x4xI20x")
        self.selections = xcffib.List(unpacker, ListItem, self.selections_len)
        self.bufsize = unpacker.offset - base


class ListSelectionsCookie(xcffib.Cookie):
    reply_type = ListSelectionsReply


class GetClientContextReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (self.context_len,) = unpacker.unpack("=xx2x4xI20x")
        self.context = xcffib.List(unpacker, "c", self.context_len)
        self.bufsize = unpacker.offset - base


class GetClientContextCookie(xcffib.Cookie):
    reply_type = GetClientContextReply


class xselinuxExtension(xcffib.Extension):
    def QueryVersion(self, client_major, client_minor, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xBB", client_major, client_minor))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, client_major, client_minor):
        return self.QueryVersion(client_major, client_minor, is_checked=True)

    def QueryVersionUnchecked(self, client_major, client_minor):
        return self.QueryVersion(client_major, client_minor, is_checked=False)

    def SetDeviceCreateContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(1, buf, is_checked=is_checked)

    def SetDeviceCreateContextChecked(self, context_len, context):
        return self.SetDeviceCreateContext(context_len, context, is_checked=True)

    def SetDeviceCreateContextUnchecked(self, context_len, context):
        return self.SetDeviceCreateContext(context_len, context, is_checked=False)

    def GetDeviceCreateContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            2, buf, GetDeviceCreateContextCookie, is_checked=is_checked
        )

    def GetDeviceCreateContextChecked(self):
        return self.GetDeviceCreateContext(is_checked=True)

    def GetDeviceCreateContextUnchecked(self):
        return self.GetDeviceCreateContext(is_checked=False)

    def SetDeviceContext(self, device, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", device, context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(3, buf, is_checked=is_checked)

    def SetDeviceContextChecked(self, device, context_len, context):
        return self.SetDeviceContext(device, context_len, context, is_checked=True)

    def SetDeviceContextUnchecked(self, device, context_len, context):
        return self.SetDeviceContext(device, context_len, context, is_checked=False)

    def GetDeviceContext(self, device, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", device))
        return self.send_request(4, buf, GetDeviceContextCookie, is_checked=is_checked)

    def GetDeviceContextChecked(self, device):
        return self.GetDeviceContext(device, is_checked=True)

    def GetDeviceContextUnchecked(self, device):
        return self.GetDeviceContext(device, is_checked=False)

    def SetWindowCreateContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(5, buf, is_checked=is_checked)

    def SetWindowCreateContextChecked(self, context_len, context):
        return self.SetWindowCreateContext(context_len, context, is_checked=True)

    def SetWindowCreateContextUnchecked(self, context_len, context):
        return self.SetWindowCreateContext(context_len, context, is_checked=False)

    def GetWindowCreateContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            6, buf, GetWindowCreateContextCookie, is_checked=is_checked
        )

    def GetWindowCreateContextChecked(self):
        return self.GetWindowCreateContext(is_checked=True)

    def GetWindowCreateContextUnchecked(self):
        return self.GetWindowCreateContext(is_checked=False)

    def GetWindowContext(self, window, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", window))
        return self.send_request(7, buf, GetWindowContextCookie, is_checked=is_checked)

    def GetWindowContextChecked(self, window):
        return self.GetWindowContext(window, is_checked=True)

    def GetWindowContextUnchecked(self, window):
        return self.GetWindowContext(window, is_checked=False)

    def SetPropertyCreateContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(8, buf, is_checked=is_checked)

    def SetPropertyCreateContextChecked(self, context_len, context):
        return self.SetPropertyCreateContext(context_len, context, is_checked=True)

    def SetPropertyCreateContextUnchecked(self, context_len, context):
        return self.SetPropertyCreateContext(context_len, context, is_checked=False)

    def GetPropertyCreateContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            9, buf, GetPropertyCreateContextCookie, is_checked=is_checked
        )

    def GetPropertyCreateContextChecked(self):
        return self.GetPropertyCreateContext(is_checked=True)

    def GetPropertyCreateContextUnchecked(self):
        return self.GetPropertyCreateContext(is_checked=False)

    def SetPropertyUseContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(10, buf, is_checked=is_checked)

    def SetPropertyUseContextChecked(self, context_len, context):
        return self.SetPropertyUseContext(context_len, context, is_checked=True)

    def SetPropertyUseContextUnchecked(self, context_len, context):
        return self.SetPropertyUseContext(context_len, context, is_checked=False)

    def GetPropertyUseContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            11, buf, GetPropertyUseContextCookie, is_checked=is_checked
        )

    def GetPropertyUseContextChecked(self):
        return self.GetPropertyUseContext(is_checked=True)

    def GetPropertyUseContextUnchecked(self):
        return self.GetPropertyUseContext(is_checked=False)

    def GetPropertyContext(self, window, property, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", window, property))
        return self.send_request(
            12, buf, GetPropertyContextCookie, is_checked=is_checked
        )

    def GetPropertyContextChecked(self, window, property):
        return self.GetPropertyContext(window, property, is_checked=True)

    def GetPropertyContextUnchecked(self, window, property):
        return self.GetPropertyContext(window, property, is_checked=False)

    def GetPropertyDataContext(self, window, property, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", window, property))
        return self.send_request(
            13, buf, GetPropertyDataContextCookie, is_checked=is_checked
        )

    def GetPropertyDataContextChecked(self, window, property):
        return self.GetPropertyDataContext(window, property, is_checked=True)

    def GetPropertyDataContextUnchecked(self, window, property):
        return self.GetPropertyDataContext(window, property, is_checked=False)

    def ListProperties(self, window, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", window))
        return self.send_request(14, buf, ListPropertiesCookie, is_checked=is_checked)

    def ListPropertiesChecked(self, window):
        return self.ListProperties(window, is_checked=True)

    def ListPropertiesUnchecked(self, window):
        return self.ListProperties(window, is_checked=False)

    def SetSelectionCreateContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(15, buf, is_checked=is_checked)

    def SetSelectionCreateContextChecked(self, context_len, context):
        return self.SetSelectionCreateContext(context_len, context, is_checked=True)

    def SetSelectionCreateContextUnchecked(self, context_len, context):
        return self.SetSelectionCreateContext(context_len, context, is_checked=False)

    def GetSelectionCreateContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            16, buf, GetSelectionCreateContextCookie, is_checked=is_checked
        )

    def GetSelectionCreateContextChecked(self):
        return self.GetSelectionCreateContext(is_checked=True)

    def GetSelectionCreateContextUnchecked(self):
        return self.GetSelectionCreateContext(is_checked=False)

    def SetSelectionUseContext(self, context_len, context, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", context_len))
        buf.write(xcffib.pack_list(context, "c"))
        return self.send_request(17, buf, is_checked=is_checked)

    def SetSelectionUseContextChecked(self, context_len, context):
        return self.SetSelectionUseContext(context_len, context, is_checked=True)

    def SetSelectionUseContextUnchecked(self, context_len, context):
        return self.SetSelectionUseContext(context_len, context, is_checked=False)

    def GetSelectionUseContext(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(
            18, buf, GetSelectionUseContextCookie, is_checked=is_checked
        )

    def GetSelectionUseContextChecked(self):
        return self.GetSelectionUseContext(is_checked=True)

    def GetSelectionUseContextUnchecked(self):
        return self.GetSelectionUseContext(is_checked=False)

    def GetSelectionContext(self, selection, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", selection))
        return self.send_request(
            19, buf, GetSelectionContextCookie, is_checked=is_checked
        )

    def GetSelectionContextChecked(self, selection):
        return self.GetSelectionContext(selection, is_checked=True)

    def GetSelectionContextUnchecked(self, selection):
        return self.GetSelectionContext(selection, is_checked=False)

    def GetSelectionDataContext(self, selection, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", selection))
        return self.send_request(
            20, buf, GetSelectionDataContextCookie, is_checked=is_checked
        )

    def GetSelectionDataContextChecked(self, selection):
        return self.GetSelectionDataContext(selection, is_checked=True)

    def GetSelectionDataContextUnchecked(self, selection):
        return self.GetSelectionDataContext(selection, is_checked=False)

    def ListSelections(self, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2x"))
        return self.send_request(21, buf, ListSelectionsCookie, is_checked=is_checked)

    def ListSelectionsChecked(self):
        return self.ListSelections(is_checked=True)

    def ListSelectionsUnchecked(self):
        return self.ListSelections(is_checked=False)

    def GetClientContext(self, resource, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", resource))
        return self.send_request(22, buf, GetClientContextCookie, is_checked=is_checked)

    def GetClientContextChecked(self, resource):
        return self.GetClientContext(resource, is_checked=True)

    def GetClientContextUnchecked(self, resource):
        return self.GetClientContext(resource, is_checked=False)


xcffib._add_ext(key, xselinuxExtension, _events, _errors)



import xcffib
import struct
import io

MAJOR_VERSION = 1
MINOR_VERSION = 1
key = xcffib.ExtensionKey("MIT-SCREEN-SAVER")
_events = {}
_errors = {}
from . import xproto


class Kind:
    Blanked = 0
    Internal = 1
    External = 2


class Event:
    NotifyMask = 1 << 0
    CycleMask = 1 << 1


class State:
    Off = 0
    On = 1
    Cycle = 2
    Disabled = 3


class QueryVersionReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        self.server_major_version, self.server_minor_version = unpacker.unpack(
            "=xx2x4xHH20x"
        )
        self.bufsize = unpacker.offset - base


class QueryVersionCookie(xcffib.Cookie):
    reply_type = QueryVersionReply


class QueryInfoReply(xcffib.Reply):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Reply.__init__(self, unpacker)
        base = unpacker.offset
        (
            self.state,
            self.saver_window,
            self.ms_until_server,
            self.ms_since_user_input,
            self.event_mask,
            self.kind,
        ) = unpacker.unpack("=xB2x4xIIIIB7x")
        self.bufsize = unpacker.offset - base


class QueryInfoCookie(xcffib.Cookie):
    reply_type = QueryInfoReply


class NotifyEvent(xcffib.Event):
    xge = False

    def __init__(self, unpacker):
        if isinstance(unpacker, xcffib.Protobj):
            unpacker = xcffib.MemoryUnpacker(unpacker.pack())
        xcffib.Event.__init__(self, unpacker)
        base = unpacker.offset
        self.state, self.time, self.root, self.window, self.kind, self.forced = (
            unpacker.unpack("=xB2xIIIBB14x")
        )
        self.bufsize = unpacker.offset - base

    def pack(self):
        buf = io.BytesIO()
        buf.write(struct.pack("=B", 0))
        buf.write(
            struct.pack(
                "=B2xIIIBB14x",
                self.state,
                self.time,
                self.root,
                self.window,
                self.kind,
                self.forced,
            )
        )
        buf_len = len(buf.getvalue())
        if buf_len < 32:
            buf.write(struct.pack(("%dx" % (32 - buf_len))))
        return buf.getvalue()

    @classmethod
    def synthetic(cls, state, time, root, window, kind, forced):
        self = cls.__new__(cls)
        self.state = state
        self.time = time
        self.root = root
        self.window = window
        self.kind = kind
        self.forced = forced
        return self


_events[0] = NotifyEvent


class screensaverExtension(xcffib.Extension):
    def QueryVersion(self, client_major_version, client_minor_version, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xBB2x", client_major_version, client_minor_version))
        return self.send_request(0, buf, QueryVersionCookie, is_checked=is_checked)

    def QueryVersionChecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=True
        )

    def QueryVersionUnchecked(self, client_major_version, client_minor_version):
        return self.QueryVersion(
            client_major_version, client_minor_version, is_checked=False
        )

    def QueryInfo(self, drawable, is_checked=True):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(1, buf, QueryInfoCookie, is_checked=is_checked)

    def QueryInfoChecked(self, drawable):
        return self.QueryInfo(drawable, is_checked=True)

    def QueryInfoUnchecked(self, drawable):
        return self.QueryInfo(drawable, is_checked=False)

    def SelectInput(self, drawable, event_mask, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xII", drawable, event_mask))
        return self.send_request(2, buf, is_checked=is_checked)

    def SelectInputChecked(self, drawable, event_mask):
        return self.SelectInput(drawable, event_mask, is_checked=True)

    def SelectInputUnchecked(self, drawable, event_mask):
        return self.SelectInput(drawable, event_mask, is_checked=False)

    def SetAttributes(
        self,
        drawable,
        x,
        y,
        width,
        height,
        border_width,
        _class,
        depth,
        visual,
        value_mask,
        value_list,
        is_checked=False,
    ):
        buf = io.BytesIO()
        buf.write(
            struct.pack(
                "=xx2xIhhHHHBBII",
                drawable,
                x,
                y,
                width,
                height,
                border_width,
                _class,
                depth,
                visual,
                value_mask,
            )
        )
        if value_mask & xproto.CW:
            background_pixmap = value_list.pop(0)
            buf.write(struct.pack("=I", background_pixmap))
        if value_mask & xproto.CW:
            background_pixel = value_list.pop(0)
            buf.write(struct.pack("=I", background_pixel))
        if value_mask & xproto.CW:
            border_pixmap = value_list.pop(0)
            buf.write(struct.pack("=I", border_pixmap))
        if value_mask & xproto.CW:
            border_pixel = value_list.pop(0)
            buf.write(struct.pack("=I", border_pixel))
        if value_mask & xproto.CW:
            bit_gravity = value_list.pop(0)
            buf.write(struct.pack("=I", bit_gravity))
        if value_mask & xproto.CW:
            win_gravity = value_list.pop(0)
            buf.write(struct.pack("=I", win_gravity))
        if value_mask & xproto.CW:
            backing_store = value_list.pop(0)
            buf.write(struct.pack("=I", backing_store))
        if value_mask & xproto.CW:
            backing_planes = value_list.pop(0)
            buf.write(struct.pack("=I", backing_planes))
        if value_mask & xproto.CW:
            backing_pixel = value_list.pop(0)
            buf.write(struct.pack("=I", backing_pixel))
        if value_mask & xproto.CW:
            override_redirect = value_list.pop(0)
            buf.write(struct.pack("=I", override_redirect))
        if value_mask & xproto.CW:
            save_under = value_list.pop(0)
            buf.write(struct.pack("=I", save_under))
        if value_mask & xproto.CW:
            event_mask = value_list.pop(0)
            buf.write(struct.pack("=I", event_mask))
        if value_mask & xproto.CW:
            do_not_propogate_mask = value_list.pop(0)
            buf.write(struct.pack("=I", do_not_propogate_mask))
        if value_mask & xproto.CW:
            colormap = value_list.pop(0)
            buf.write(struct.pack("=I", colormap))
        if value_mask & xproto.CW:
            cursor = value_list.pop(0)
            buf.write(struct.pack("=I", cursor))
        return self.send_request(3, buf, is_checked=is_checked)

    def SetAttributesChecked(
        self,
        drawable,
        x,
        y,
        width,
        height,
        border_width,
        _class,
        depth,
        visual,
        value_mask,
        value_list,
    ):
        return self.SetAttributes(
            drawable,
            x,
            y,
            width,
            height,
            border_width,
            _class,
            depth,
            visual,
            value_mask,
            value_list,
            is_checked=True,
        )

    def SetAttributesUnchecked(
        self,
        drawable,
        x,
        y,
        width,
        height,
        border_width,
        _class,
        depth,
        visual,
        value_mask,
        value_list,
    ):
        return self.SetAttributes(
            drawable,
            x,
            y,
            width,
            height,
            border_width,
            _class,
            depth,
            visual,
            value_mask,
            value_list,
            is_checked=False,
        )

    def UnsetAttributes(self, drawable, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", drawable))
        return self.send_request(4, buf, is_checked=is_checked)

    def UnsetAttributesChecked(self, drawable):
        return self.UnsetAttributes(drawable, is_checked=True)

    def UnsetAttributesUnchecked(self, drawable):
        return self.UnsetAttributes(drawable, is_checked=False)

    def Suspend(self, suspend, is_checked=False):
        buf = io.BytesIO()
        buf.write(struct.pack("=xx2xI", suspend))
        return self.send_request(5, buf, is_checked=is_checked)

    def SuspendChecked(self, suspend):
        return self.Suspend(suspend, is_checked=True)

    def SuspendUnchecked(self, suspend):
        return self.Suspend(suspend, is_checked=False)


xcffib._add_ext(key, screensaverExtension, _events, _errors)
